package org.javafxdata.datasources.provider;

import javafx.collections.ObservableList;

/**
 *
 * A ListProvider is capable of providing an ObservableList to a consumer
 * (e.g. a ListView). 
 *
 * @author johan
 */
public interface ListProvider<T> extends ObjectProvider{
    
     /**
     * Return the items that are obtained by this provider in an
     * ObservableList. After this method has been called, items can still be 
     * added to the returned ObservableList by the provider.<br/>
     * This method should return immediately, even when the incoming data is not
     * completely processed yet. When the processing of the incoming data leads
     * to new items or deleted items in the returned ObservableList, the consumer of the
     * ObservableList can be notified.
     * 
     * @return an instance of an implementation of ObservableList that contains
     * or will contain the data retrieved by this Provider
     */
    public ObservableList<T> getData();
    
}
