package org.javafxdata.datasources.provider;

import javafx.beans.Observable;

/**
 * An ObjectProvider is capable of providing an Observable Object to a consumer
 * (e.g. a JavaFX Control). 
 *
 * @author johan
 */
public interface ObjectProvider {
    /**
     * Return the Object that is obtained by this provider. The return Object
     * is an implementation of Observable, hence its value can change after this
     * method has been called. <br/>
     * This method should return immediately, even when the incoming data is not
     * completely processed yet. When the processing of the incoming data leads
     * to changing values of the returned Observable, the consumer of the
     * ObservableValue can be notified.
     * 
     * @return an instance of an implementation of Observable that contains
     * or will contain the data retrieved by this Provider
     */
    Observable getData();
    
}