package org.javafxdata.datasources.provider;

import javafx.collections.ObservableList;
import javafx.scene.control.TableColumn;

/**
 *
 * A ListProvider is capable of providing an ObservableList to a consumer, where
 * the items in the ObservableList can be used to create columns in a table 
 * structure (e.g. TableView).
 * @author johan
 */
public interface TableProvider<T> extends ListProvider<T> {
  
    /** 
     * Define which columns should be created by this TableProvider.
     * The matching between the column names and the TableColumns that will be
     * returned by <code>getColumns</code> is protocol-dependent. 
     * 
     * @param columns 
     */
    void setColumnNames (String... columns);
    
    /**
     * This method returns an ObservableList containing the different TableColumn
     * instances for this provider. 
     * <br/>
     * This method should return immediately, even when the provider has not created
     * all TableColumn instances yet -- other instances can be added later to 
     * the ObservableList by the provider.
     * <br/>
     * It is important to note that the instances of TableColumn are 
     * responsible for returning the correct column data.
     * @return an ObservableList containing the TableColumns for this provider.
     */
    ObservableList<TableColumn<T,?>> getColumns();
    
}
