package org.javafxdata.datasources.provider;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.util.Callback;

/**
 *
 * @author johan
 */
public abstract class TableProviderBase<T> extends ListProviderBase<T> implements TableProvider {

    protected List<String> columnNames = new LinkedList<String>();
    private ObservableList<TableColumn<T, ?>> tableColumns = FXCollections.observableArrayList();

    @Override
    public ObservableList<TableColumn<T, ?>> getColumns() {
        return tableColumns;
    }

    @Override
    public void setColumnNames(String... columns) {
        this.columnNames = Arrays.asList(columns);
    }

    protected void buildColumns() {
        for (final String columnName : columnNames) {
            TableColumn<T, Object> tc = new TableColumn<T, Object>(columnName);
            tc.setCellValueFactory(new Callback<TableColumn.CellDataFeatures<T, Object>, ObservableValue<Object>>() {

                @Override
                public ObservableValue<Object> call(CellDataFeatures<T, Object> cdf) {
                    T myRow = (T) cdf.getValue();
                    return (ObservableValue<Object>) new ReadOnlyObjectWrapper(getMyElement(myRow, columnName));
                }
            });
            tableColumns.add(tc);
//            super.addColumn(columnName, tc);


        }
    }

    protected abstract Object getMyElement(T myRow, String columnName);
}