package org.javafxdata.datasources.reader;

import java.io.IOException;
import java.io.InputStream;
import org.javafxdata.datasources.Format;

/**
 *
 * This is the interface that is implemented by a number of physical datasources.
 * It is used by protocol implementations that rely on this interface to retrieve
 * the actual data.<br/>
 * Implementations of this interface should not deal with what parts of the
 * data that have to be rendered.
 * 
 * @author johan
 */
public interface DataSourceReader {
    
    /**
     * return an InputStream that can be read by protocol implementations and 
     * that contain the data. The format of the data on this InputStream is
     * not specified.
     * 
     * @return an InputStream containing data
     */
    public InputStream getInputStream() throws IOException;
    
    /**
     * Return the format of the data in this DataSourceReader.  
     * @return the format of the data
     */
    public Format getInputFormat();
    
}
