package org.javafxdata.datasources.reader;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import org.javafxdata.datasources.Format;

/**
 *
 * Implementation of DataSourceReader for (local) filesystems. Instances of this
 * class can be created and passed in the constructor of a protocol
 * implementations in the org.javafxdata.datasources.io package.
 *
 * @author johan
 */
public class FileSource implements DataSourceReader {

    private InputStream is;
    private Format format = Format.XML;

    /**
     * Create a FileSource based on an existing file.
     * By default, it is assumed the input format is XML.
     * Use FileSource (String, Format) to specify a different format.
     *
     * @param filename the filename
     * @throws FileNotFoundException
     */
    public FileSource(String filename) throws FileNotFoundException {
        this(filename, Format.XML);
    }
    
    /**
     * Create a FileSource based on an existing file.
     * By default, it is assumed the input format is XML.
     * Use FileSource (String, Format) to specify a different format.
     *
     * @param filename the filename
     * @throws FileNotFoundException
     */
    public FileSource(File f) throws FileNotFoundException {
        this(f, Format.XML);
    }
    
    /**
     * Create a FileSource based on an existing inputstream.
     * By default, it is assumed the input format is XML.
     * Use FileSource (String, Format) to specify a different format.
     *
     * @param filename the filename
     * @throws FileNotFoundException
     */
    public FileSource(InputStream is) {
        this(is, Format.XML);
    }
    
    /**
     * Create a FileSource based on an existing file.
     *
     * @param filename the filename
     * @param format the format of the data
     * @throws FileNotFoundException
     */
    public FileSource(String filename, Format format) throws FileNotFoundException {
        this.is = new FileInputStream(filename);
        this.format = format;
    }
    
    /**
     * Create a FileSource based on an existing file.
     *
     * @param f the file
     * @param format the format of the data
     * @throws FileNotFoundException
     */
    public FileSource(File f, Format format) throws FileNotFoundException {
        this.is = new FileInputStream(f);
        this.format = format;

    }
    /**
     * Create a FileSource based on an existing inputstream.
     *
     * @param filename the filename
     * @param format the format of the data
     * @throws FileNotFoundException
     */
    public FileSource(InputStream is, Format format) {
        this.is = is;
        this.format = format;

    }

    @Override
    public InputStream getInputStream() {
        return this.is;
    }

    @Override
    public Format getInputFormat() {
        return this.format;
    }
}
